﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ship
{
    internal class MapWithSetShipsGeneric<T, U>
        where T : class, IDrawingObject
        where U : AbstractMap
    {
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Размер занимаемого объектом места (ширина)
        /// </summary>
        private readonly int _placeSizeWidth = 210;
        /// <summary>
        /// Размер занимаемого объектом места (высота)
        /// </summary>
        private readonly int _placeSizeHeight = 90;
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly SetShipsGeneric<T> _setShips;
        /// <summary>
        /// Карта
        /// </summary>
        private readonly U _map;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="picWidth"></param>
        /// <param name="picHeight"></param>
        /// <param name="map"></param>
        public MapWithSetShipsGeneric(int picWidth, int picHeight, U map)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _setShips = new SetShipsGeneric<T>(width * height);
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _map = map;
        }
        /// <summary>
        /// Перегрузка оператора сложения
        /// </summary>
        /// <param name="map"></param>
        /// <param name="car"></param>
        /// <returns></returns>
        public static int operator +(MapWithSetShipsGeneric<T, U> map, T car)
        {
            return map._setShips.Insert(car);
        }
        /// <summary>
        /// Перегрузка оператора вычитания
        /// </summary>
        /// <param name="map"></param>
        /// <param name="position"></param>
        /// <returns></returns>
        public static T operator -(MapWithSetShipsGeneric<T, U> map, int
       position)
        {
            return map._setShips.Remove(position);
        }
        /// <summary>
        /// Вывод всего набора объектов
        /// </summary>
        /// <returns></returns>
        public Bitmap ShowSet()
        {
            Shaking();
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawShips(gr);
            return bmp;
        }
        /// <summary>
        /// Просмотр объекта на карте
        /// </summary>
        /// <returns></returns>
        public Bitmap ShowOnMap()
        {
            Shaking();
            for (int i = 0; i < _setShips.Count; i++)
            {
                var car = _setShips[i];
                if (car != null)
                {
                    return _map.CreateMap(_pictureWidth, _pictureHeight, car);
                }
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// <summary>
        /// Перемещение объекта по крате
        /// </summary>
        /// <param name="direction"></param>
        /// <returns></returns>
        public Bitmap MoveObject(Direction direction)
        {
            if (_map != null)
            {
                return _map.MoveObject(direction);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// <summary>
        /// "Взбалтываем" набор, чтобы все элементы оказались в начале
        /// </summary>
        private void Shaking()
        {
            int j = _setShips.Count - 1;
            for (int i = 0; i < _setShips.Count; i++)
            {
                if (_setShips[i] == null)
                {
                    for (; j > i; j--)
                    {
                        var car = _setShips[j];
                        if (car != null)
                        {
                            _setShips.Insert(car, i);
                            _setShips.Remove(j);
                            break;
                        }
                    }
                    if (j <= i)
                    {
                        return;
                    }
                }
            }
        }
        /// <summary>
        /// Метод отрисовки фона
        /// </summary>
        /// <param name="g"></param>
        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            Brush seaBrush = new SolidBrush(Color.Blue);
            Pen thickPen = new Pen(Color.Black, 8);
            Pen thinPen = new Pen(Color.Black, 4);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
                {//линия рамзетки места
                    g.DrawLine(pen, i * _placeSizeWidth, j * _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j * _placeSizeHeight);
                    g.FillRectangle(seaBrush, i * _placeSizeWidth, j * _placeSizeHeight + _placeSizeHeight * 3 / 4 - 5, _placeSizeWidth / 2, _placeSizeHeight / 3);
                    g.DrawLine(thickPen, i * _placeSizeWidth, j * _placeSizeHeight - 45, i * _placeSizeWidth + _placeSizeWidth / 2 - 50, j * _placeSizeHeight - 45);
                    g.DrawLine(thinPen, i * _placeSizeWidth + 10, j * _placeSizeHeight - 45, i * _placeSizeWidth + 10, j * _placeSizeHeight - 60);
                    g.DrawLine(thinPen, i * _placeSizeWidth + 20, j * _placeSizeHeight - 45, i * _placeSizeWidth + 20, j * _placeSizeHeight - 60);
                    g.DrawLine(thinPen, i * _placeSizeWidth + 30, j * _placeSizeHeight - 45, i * _placeSizeWidth + 30, j * _placeSizeHeight - 60);
                    g.DrawLine(thinPen, i * _placeSizeWidth + 40, j * _placeSizeHeight - 45, i * _placeSizeWidth + 40, j * _placeSizeHeight - 60);
                }
                g.DrawLine(pen, i * _placeSizeWidth, 0, i * _placeSizeWidth, (_pictureHeight / _placeSizeHeight) * _placeSizeHeight);
            }
        }
        /// <summary>
        /// Метод прорисовки объектов
        /// </summary>
        /// <param name="g"></param>
        private void DrawShips(Graphics g)
        {
            int width = _pictureWidth / _placeSizeWidth;
            int height = _pictureHeight / _placeSizeHeight;

            int i = 0;
            foreach (var ship in _setShips.GetShips())
            {
                ship.SetObject((width - i % width - 1) * _placeSizeWidth + 10, (i / width) * _placeSizeHeight + 10, _pictureWidth, _pictureHeight);
                ship.DrawningObject(g);
                i++;
            }
        }
    }
}
