﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ship
{
    internal class SetShipsGeneric<T>
        where T: class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Count;

        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetShipsGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="ship">Добавляемый корабль</param>
        /// <returns></returns>
        public int Insert(T ship)
        {
            return Insert(ship, 0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="ship">Добавляемый корабль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T ship, int position)
        {
            if (position > _maxCount || Count == _maxCount || position < 0) return -1;

            _places.Insert(position, ship);
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position < 0 || position >= Count)
            {
                return null;
            }

            var result = _places[position];

            _places.RemoveAt(position);
            return result;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            get
            {
                if (position >= Count || position < 0)
                {
                    return null;
                }
                return _places[position];
            }
            set
            {
                if (position < Count && position >= 0)
                {
                    Insert(value, position);
                }
            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetShips()
        {
            foreach (var ship in _places)
            {
                if (ship != null)
                {
                    yield return ship;
                }
                else
                {
                    yield break;
                }
            }
        }
    }
}
